/**
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 *  
 */
package org.emftext.language.primitivetypes.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import org.eclipse.emf.ecore.util.Switch;

import org.emftext.language.primitivetypes.*;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see org.emftext.language.primitivetypes.PrimitivetypesPackage
 * @generated
 */
public class PrimitivetypesSwitch<T> extends Switch<T>
{
  /**
   * The cached model package
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected static PrimitivetypesPackage modelPackage;

  /**
   * Creates an instance of the switch.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  public PrimitivetypesSwitch()
  {
    if (modelPackage == null)
    {
      modelPackage = PrimitivetypesPackage.eINSTANCE;
    }
  }

  /**
   * Checks whether this is a switch for the given package.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @parameter ePackage the package in question.
   * @return whether this is a switch for the given package.
   * @generated
   */
  @Override
  protected boolean isSwitchFor(EPackage ePackage)
  {
    return ePackage == modelPackage;
  }

  /**
   * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @return the first non-null result returned by a <code>caseXXX</code> call.
   * @generated
   */
  @Override
  protected T doSwitch(int classifierID, EObject theEObject)
  {
    switch (classifierID)
    {
      case PrimitivetypesPackage.ESTRING_ELEMENT:
      {
        EStringElement eStringElement = (EStringElement)theEObject;
        T result = caseEStringElement(eStringElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case PrimitivetypesPackage.EBOOLEAN_ELEMENT:
      {
        EBooleanElement eBooleanElement = (EBooleanElement)theEObject;
        T result = caseEBooleanElement(eBooleanElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case PrimitivetypesPackage.ECHAR_ELEMENT:
      {
        ECharElement eCharElement = (ECharElement)theEObject;
        T result = caseECharElement(eCharElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case PrimitivetypesPackage.EFLOAT_ELEMENT:
      {
        EFloatElement eFloatElement = (EFloatElement)theEObject;
        T result = caseEFloatElement(eFloatElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case PrimitivetypesPackage.EDOUBLE_ELEMENT:
      {
        EDoubleElement eDoubleElement = (EDoubleElement)theEObject;
        T result = caseEDoubleElement(eDoubleElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case PrimitivetypesPackage.EINT_ELEMENT:
      {
        EIntElement eIntElement = (EIntElement)theEObject;
        T result = caseEIntElement(eIntElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case PrimitivetypesPackage.EBIG_INTEGER_ELEMENT:
      {
        EBigIntegerElement eBigIntegerElement = (EBigIntegerElement)theEObject;
        T result = caseEBigIntegerElement(eBigIntegerElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      default: return defaultCase(theEObject);
    }
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EString Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EString Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseEStringElement(EStringElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EBoolean Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EBoolean Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseEBooleanElement(EBooleanElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EChar Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EChar Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseECharElement(ECharElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EFloat Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EFloat Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseEFloatElement(EFloatElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EDouble Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EDouble Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseEDoubleElement(EDoubleElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EInt Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EInt Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseEIntElement(EIntElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EBig Integer Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EBig Integer Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseEBigIntegerElement(EBigIntegerElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch, but this is the last case anyway.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject)
   * @generated
   */
  @Override
  public T defaultCase(EObject object)
  {
    return null;
  }

} //PrimitivetypesSwitch
