/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.templateconcepts.call.resource.templatecall.mopp;

public class TemplatecallPrinter implements org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextPrinter {
	
	protected org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolverFactory tokenResolverFactory = new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallTokenResolverFactory();
	
	protected java.io.OutputStream outputStream;
	
	/**
	 * Holds the resource that is associated with this printer. This may be null if
	 * the printer is used stand alone.
	 */
	private org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextResource resource;
	
	private java.util.Map<?, ?> options;
	private String encoding = System.getProperty("file.encoding");
	
	public TemplatecallPrinter(java.io.OutputStream outputStream, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextResource resource) {
		super();
		this.outputStream = outputStream;
		this.resource = resource;
	}
	
	protected int matchCount(java.util.Map<String, Integer> featureCounter, java.util.Collection<String> needed) {
		int pos = 0;
		int neg = 0;
		
		for (String featureName : featureCounter.keySet()) {
			if (needed.contains(featureName)) {
				int value = featureCounter.get(featureName);
				if (value == 0) {
					neg += 1;
				} else {
					pos += 1;
				}
			}
		}
		return neg > 0 ? -neg : pos;
	}
	
	protected void doPrint(org.eclipse.emf.ecore.EObject element, java.io.PrintWriter out, String globaltab) {
		if (element == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write.");
		}
		if (out == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write on.");
		}
		
		if (element instanceof org.emftext.language.templateconcepts.call.TemplateCall) {
			print_org_emftext_language_templateconcepts_call_TemplateCall((org.emftext.language.templateconcepts.call.TemplateCall) element, globaltab, out);
			return;
		}
		
		addWarningToResource("The printer can not handle " + element.eClass().getName() + " elements", element);
	}
	
	protected org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallReferenceResolverSwitch getReferenceResolverSwitch() {
		return (org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallReferenceResolverSwitch) new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallMetaInformation().getReferenceResolverSwitch();
	}
	
	protected void addWarningToResource(final String errorMessage, org.eclipse.emf.ecore.EObject cause) {
		org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextResource resource = getResource();
		if (resource == null) {
			// the resource can be null if the printer is used stand alone
			return;
		}
		resource.addProblem(new org.emftext.language.templateconcepts.call.resource.templatecall.mopp.TemplatecallProblem(errorMessage, org.emftext.language.templateconcepts.call.resource.templatecall.TemplatecallEProblemType.PRINT_PROBLEM, org.emftext.language.templateconcepts.call.resource.templatecall.TemplatecallEProblemSeverity.WARNING), cause);
	}
	
	public void setOptions(java.util.Map<?,?> options) {
		this.options = options;
	}
	
	public java.util.Map<?,?> getOptions() {
		return options;
	}
	
	public void setEncoding(String encoding) {
		if (encoding != null) {
			this.encoding = encoding;
		}
	}
	
	public String getEncoding() {
		return encoding;
	}
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTextResource getResource() {
		return resource;
	}
	
	/**
	 * Calls {@link #doPrint(EObject, PrintWriter, String)} and writes the result to
	 * the underlying output stream.
	 */
	public void print(org.eclipse.emf.ecore.EObject element) throws java.io.IOException {
		java.io.PrintWriter out = new java.io.PrintWriter(new java.io.OutputStreamWriter(new java.io.BufferedOutputStream(outputStream), encoding));
		doPrint(element, out, "");
		out.flush();
		out.close();
	}
	
	public void print_org_emftext_language_templateconcepts_call_TemplateCall(org.emftext.language.templateconcepts.call.TemplateCall element, String outertab, java.io.PrintWriter out) {
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(2);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__TARGET));
		printCountingMap.put("target", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__PARAMETER_MODEL));
		printCountingMap.put("parameterModel", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("CALL");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("target");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__TARGET));
			if (o != null) {
				org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_34_34");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getTemplateCallTargetReferenceResolver().deResolve((org.emftext.language.templateconcepts.Template) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__TARGET)), element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__TARGET), element));
				out.print(" ");
			}
			printCountingMap.put("target", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("USING");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("parameterModel");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__PARAMETER_MODEL));
			if (o != null) {
				org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_34_34");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getTemplateCallParameterModelReferenceResolver().deResolve((org.eclipse.emf.ecore.EObject) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__PARAMETER_MODEL)), element.eClass().getEStructuralFeature(org.emftext.language.templateconcepts.call.CallPackage.TEMPLATE_CALL__PARAMETER_MODEL), element));
				out.print(" ");
			}
			printCountingMap.put("parameterModel", count - 1);
		}
	}
	
	
}
