/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.templateconcepts.call.resource.templatecall.mopp;

public class TemplatecallReferenceResolverSwitch implements org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolverSwitch {
	
	/**
	 * This map stores a copy of the options the were set for loading the resource.
	 */
	private java.util.Map<Object, Object> options;
	
	protected org.emftext.language.templateconcepts.call.resource.templatecall.analysis.TemplateCallTargetReferenceResolver templateCallTargetReferenceResolver = new org.emftext.language.templateconcepts.call.resource.templatecall.analysis.TemplateCallTargetReferenceResolver();
	protected org.emftext.language.templateconcepts.call.resource.templatecall.analysis.TemplateCallParameterModelReferenceResolver templateCallParameterModelReferenceResolver = new org.emftext.language.templateconcepts.call.resource.templatecall.analysis.TemplateCallParameterModelReferenceResolver();
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver<org.emftext.language.templateconcepts.call.TemplateCall, org.emftext.language.templateconcepts.Template> getTemplateCallTargetReferenceResolver() {
		return getResolverChain(org.emftext.language.templateconcepts.call.CallPackage.eINSTANCE.getTemplateCall_Target(), templateCallTargetReferenceResolver);
	}
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver<org.emftext.language.templateconcepts.call.TemplateCall, org.eclipse.emf.ecore.EObject> getTemplateCallParameterModelReferenceResolver() {
		return getResolverChain(org.emftext.language.templateconcepts.call.CallPackage.eINSTANCE.getTemplateCall_ParameterModel(), templateCallParameterModelReferenceResolver);
	}
	
	public void setOptions(java.util.Map<?, ?> options) {
		if (options != null) {
			this.options = new java.util.LinkedHashMap<Object, Object>();
			this.options.putAll(options);
		}
		templateCallTargetReferenceResolver.setOptions(options);
		templateCallParameterModelReferenceResolver.setOptions(options);
	}
	
	public void resolveFuzzy(String identifier, org.eclipse.emf.ecore.EObject container, org.eclipse.emf.ecore.EReference reference, int position, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolveResult<org.eclipse.emf.ecore.EObject> result) {
		if (container == null) {
			return;
		}
		if (org.emftext.language.templateconcepts.call.CallPackage.eINSTANCE.getTemplateCall().isInstance(container)) {
			TemplatecallFuzzyResolveResult<org.emftext.language.templateconcepts.Template> frr = new TemplatecallFuzzyResolveResult<org.emftext.language.templateconcepts.Template>(result);
			String referenceName = reference.getName();
			org.eclipse.emf.ecore.EStructuralFeature feature = container.eClass().getEStructuralFeature(referenceName);
			if (feature != null && feature instanceof org.eclipse.emf.ecore.EReference && referenceName != null && referenceName.equals("target")) {
				templateCallTargetReferenceResolver.resolve(identifier, (org.emftext.language.templateconcepts.call.TemplateCall) container, (org.eclipse.emf.ecore.EReference) feature, position, true, frr);
			}
		}
		if (org.emftext.language.templateconcepts.call.CallPackage.eINSTANCE.getTemplateCall().isInstance(container)) {
			TemplatecallFuzzyResolveResult<org.eclipse.emf.ecore.EObject> frr = new TemplatecallFuzzyResolveResult<org.eclipse.emf.ecore.EObject>(result);
			String referenceName = reference.getName();
			org.eclipse.emf.ecore.EStructuralFeature feature = container.eClass().getEStructuralFeature(referenceName);
			if (feature != null && feature instanceof org.eclipse.emf.ecore.EReference && referenceName != null && referenceName.equals("parameterModel")) {
				templateCallParameterModelReferenceResolver.resolve(identifier, (org.emftext.language.templateconcepts.call.TemplateCall) container, (org.eclipse.emf.ecore.EReference) feature, position, true, frr);
			}
		}
	}
	
	public org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver<? extends org.eclipse.emf.ecore.EObject, ? extends org.eclipse.emf.ecore.EObject> getResolver(org.eclipse.emf.ecore.EStructuralFeature reference) {
		if (reference == org.emftext.language.templateconcepts.call.CallPackage.eINSTANCE.getTemplateCall_Target()) {
			return getResolverChain(reference, templateCallTargetReferenceResolver);
		}
		if (reference == org.emftext.language.templateconcepts.call.CallPackage.eINSTANCE.getTemplateCall_ParameterModel()) {
			return getResolverChain(reference, templateCallParameterModelReferenceResolver);
		}
		return null;
	}
	
	@SuppressWarnings({"rawtypes", "unchecked"})	
	public <ContainerType extends org.eclipse.emf.ecore.EObject, ReferenceType extends org.eclipse.emf.ecore.EObject> org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver<ContainerType, ReferenceType> getResolverChain(org.eclipse.emf.ecore.EStructuralFeature reference, org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver<ContainerType, ReferenceType> originalResolver) {
		if (options == null) {
			return originalResolver;
		}
		Object value = options.get(org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallOptions.ADDITIONAL_REFERENCE_RESOLVERS);
		if (value == null) {
			return originalResolver;
		}
		if (!(value instanceof java.util.Map)) {
			// send this to the error log
			new org.emftext.language.templateconcepts.call.resource.templatecall.util.TemplatecallRuntimeUtil().logWarning("Found value with invalid type for option " + org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallOptions.ADDITIONAL_REFERENCE_RESOLVERS + " (expected " + java.util.Map.class.getName() + ", but was " + value.getClass().getName() + ")", null);
			return originalResolver;
		}
		java.util.Map<?,?> resolverMap = (java.util.Map<?,?>) value;
		Object resolverValue = resolverMap.get(reference);
		if (resolverValue == null) {
			return originalResolver;
		}
		if (resolverValue instanceof org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver) {
			org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver replacingResolver = (org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver) resolverValue;
			if (replacingResolver instanceof org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallDelegatingReferenceResolver) {
				// pass original resolver to the replacing one
				((org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallDelegatingReferenceResolver) replacingResolver).setDelegate(originalResolver);
			}
			return replacingResolver;
		} else if (resolverValue instanceof java.util.Collection) {
			java.util.Collection replacingResolvers = (java.util.Collection) resolverValue;
			org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver replacingResolver = originalResolver;
			for (Object next : replacingResolvers) {
				if (next instanceof org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceCache) {
					org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver nextResolver = (org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallReferenceResolver) next;
					if (nextResolver instanceof org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallDelegatingReferenceResolver) {
						// pass original resolver to the replacing one
						((org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallDelegatingReferenceResolver) nextResolver).setDelegate(replacingResolver);
					}
					replacingResolver = nextResolver;
				} else {
					// The collection contains a non-resolver. Send a warning to the error log.
					new org.emftext.language.templateconcepts.call.resource.templatecall.util.TemplatecallRuntimeUtil().logWarning("Found value with invalid type in value map for option " + org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallOptions.ADDITIONAL_REFERENCE_RESOLVERS + " (expected " + org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallDelegatingReferenceResolver.class.getName() + ", but was " + next.getClass().getName() + ")", null);
				}
			}
			return replacingResolver;
		} else {
			// The value for the option ADDITIONAL_REFERENCE_RESOLVERS has an unknown type.
			new org.emftext.language.templateconcepts.call.resource.templatecall.util.TemplatecallRuntimeUtil().logWarning("Found value with invalid type in value map for option " + org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallOptions.ADDITIONAL_REFERENCE_RESOLVERS + " (expected " + org.emftext.language.templateconcepts.call.resource.templatecall.ITemplatecallDelegatingReferenceResolver.class.getName() + ", but was " + resolverValue.getClass().getName() + ")", null);
			return originalResolver;
		}
	}
	
}
